import re


class Auxiliar:
    @staticmethod
    def format_as_asterisk_command(asterisk_command: str) -> str:
        result = asterisk_command.replace("\n", "\r\n")
        return re.sub("[ ]{2,}", "", result)



from typing import Iterable


class Action(object):
    def __init__(self, name, keys=None, variables=None):
        self.name = name
        self.keys = keys or dict()
        if not isinstance(variables, Iterable):
            variables = dict()
        self.variables = variables or dict()

    def __str__(self):
        package = "Action: %s\r\n" % self.name
        for key in self.keys:
            if key != "Action" and key != "Variables":
                package += "%s: %s\r\n" % (key, self.keys[key])
        for var in self.variables:
            package += "Variable: %s=%s\r\n" % (var, self.variables[var])
        return package

    def __getattr__(self, item):
        return self.keys[item]

    def __setattr__(self, key, value):
        if key in ("name", "keys", "variables"):
            return object.__setattr__(self, key, value)
        self.keys[key] = value

    def __setitem__(self, key, value):
        self.variables[key] = value

    def __getitem__(self, item):
        return self.variables[item]



from unittest import TestCase
from uuid import uuid4
from AsteriskCommand.queue_pause import QueuePause
from tests.commands.auxiliar_commands import Auxiliar


class TestCommands(TestCase):
    def test_queue_pause(self):
        action_id = str(uuid4())

        asterisk_command = QueuePause(
            ActionID=action_id,
            Queue="ventas",
            Interface="SIP/999",
            Paused=False,
            Reason="100001",
            MemberName="1000",
        )

        expected_dictionary = {
            "Action": "QueuePause",
            "ActionID": action_id,
            "Queue": "ventas",
            "Interface": "SIP/999",
            "Paused": False,
            "Reason": "100001",
            "MemberName": "1000",
        }

        expected_asterisk_command = Auxiliar.format_as_asterisk_command(
            str(
                f"""Action: QueuePause
                ActionID: {action_id}
                Queue: ventas
                Interface: SIP/999
                Paused: False
                Reason: 100001
                MemberName: 1000
                """
            )
        )

        self.assertDictEqual(asterisk_command.as_dict(), expected_dictionary)
        self.assertEqual(
            str(asterisk_command.as_asterisk_command()), expected_asterisk_command
        )
        self.assertTrue(asterisk_command.is_asterisk_command)



from unittest import TestCase
import json
from uuid import uuid4
from AsteriskCommand.atxfer import Atxfer
from tests.commands.auxiliar_commands import Auxiliar


class TestCommands(TestCase):
    def test_atxfer(self):
        action_id = str(uuid4())

        command = Atxfer(
            ActionID=action_id,
            Channel="SIP/123",
            Context="Test",
            Exten="SIP/111",
            Priority="1",
            peerName="1000",
        )

        expected_dictionary = {
            "Action": "Atxfer",
            "ActionID": action_id,
            "Channel": "SIP/123",
            "Context": "Test",
            "Exten": "SIP/111",
            "Priority": "1",
            "peerName": "1000",
        }

        expected_asterisk_command = Auxiliar.format_as_asterisk_command(
            str(
                f"""Action: Atxfer
                ActionID: {action_id}
                Channel: SIP/123
                Context: Test
                Exten: SIP/111
                Priority: 1
                peerName: 1000
                """
            )
        )

        self.assertDictEqual(command.as_dict(), expected_dictionary)
        self.assertTrue(command.is_asterisk_command)
        self.assertEqual(str(command.as_asterisk_command()), expected_asterisk_command)

        self.assertEqual(
            command.as_json_asterisk_command(), json.dumps(expected_dictionary)
        )



from typing import Dict
from dataclasses import dataclass
from AsteriskCommand.commons.asterisk_command import AsteriskCommand


@dataclass
class ClearIdleCallActors(AsteriskCommand):
    ActionID: str
    Action: str = "ClearIdleCallActors"
    is_asterisk_command: bool = False

    def as_dict(self) -> Dict:
        return {
            "Action": self.Action,
            "ActionID": self.ActionID,
        }



from typing import Dict
from dataclasses import dataclass
from AsteriskCommand.commons.asterisk_command import AsteriskCommand


@dataclass
class Atxfer(AsteriskCommand):
    ActionID: str
    Channel: str
    Context: str
    Exten: str
    Priority: str
    peerName: str
    Action: str = "Atxfer"

    def as_dict(self) -> Dict:
        return {
            "Action": self.Action,
            "ActionID": self.ActionID,
            "Channel": self.Channel,
            "Context": self.Context,
            "Exten": self.Exten,
            "Priority": self.Priority,
            "peerName": self.peerName,
        }



from typing import Dict
from dataclasses import dataclass, field
from AsteriskCommand.commons.asterisk_command import AsteriskCommand


@dataclass
class Originate(AsteriskCommand):
    ActionID: str
    Channel: str
    Context: str
    Exten: str
    Priority: str
    Callerid: str
    Timeout: str
    Variables: Dict = field(default_factory=dict)
    Action: str = "Originate"

    def as_dict(self) -> Dict:
        return {
            "Action": self.Action,
            "ActionID": self.ActionID,
            "Channel": self.Channel,
            "Context": self.Context,
            "Exten": self.Exten,
            "Priority": self.Priority,
            "Callerid": self.Callerid,
            "Timeout": self.Timeout,
            "Variables": self.Variables,
        }



from unittest import TestCase
import json
from uuid import uuid4
from AsteriskCommand.originate import Originate
from tests.commands.auxiliar_commands import Auxiliar


class TestCommands(TestCase):
    def test_originate(self):
        action_id = str(uuid4())
        transaction_id = str(uuid4())

        command = Originate(
            ActionID=action_id,
            Channel="SIP/100",
            Context="Test",
            Exten="999",
            Priority="1",
            Callerid="10000",
            Timeout=30,
        )
        command.Variables = {
            "IDRegistro": "100",
            "CallActionID": transaction_id,
            "AgentCode": "999",
        }

        expected_dictionary = {
            "Action": "Originate",
            "ActionID": action_id,
            "Channel": "SIP/100",
            "Context": "Test",
            "Exten": "999",
            "Priority": "1",
            "Callerid": "10000",
            "Timeout": 30,
            "Variables": {
                "IDRegistro": "100",
                "CallActionID": transaction_id,
                "AgentCode": "999",
            },
        }

        expected_asterisk_command = Auxiliar.format_as_asterisk_command(
            str(
                f"""Action: Originate
                ActionID: {action_id}
                Channel: SIP/100
                Context: Test
                Exten: 999
                Priority: 1
                Callerid: 10000
                Timeout: 30
                Variable: IDRegistro=100
                Variable: CallActionID={transaction_id}
                Variable: AgentCode=999
                """
            )
        )

        self.assertTrue(command.is_asterisk_command)
        self.assertDictEqual(command.as_dict(), expected_dictionary)
        self.assertEqual(
            command.as_json_asterisk_command(), json.dumps(expected_dictionary)
        )
        self.assertEqual(str(command.as_asterisk_command()), expected_asterisk_command)



from unittest import TestCase
from uuid import uuid4
from AsteriskCommand.redirect import Redirect
from tests.commands.auxiliar_commands import Auxiliar


class TestCommands(TestCase):
    def test_redirect(self):
        action_id = str(uuid4())

        asterisk_command = Redirect(
            ActionID=action_id,
            Context="redirect",
            Exten="123",
            Priority="1",
            Channel="SIP/999",
            peerName="SIP/888",
        )

        expected_dictionary = {
            "Action": "Redirect",
            "ActionID": action_id,
            "Context": "redirect",
            "Exten": "123",
            "Priority": "1",
            "Channel": "SIP/999",
            "peerName": "SIP/888",
        }

        expected_asterisk_command = Auxiliar.format_as_asterisk_command(
            str(
                f"""Action: Redirect
                ActionID: {action_id}
                Channel: SIP/999
                Context: redirect
                Exten: 123
                Priority: 1
                peerName: SIP/888
                """
            )
        )

        self.assertDictEqual(asterisk_command.as_dict(), expected_dictionary)
        self.assertEqual(
            str(asterisk_command.as_asterisk_command()), expected_asterisk_command
        )
        self.assertTrue(asterisk_command.is_asterisk_command)



import json
from typing import Dict, List
from abc import ABC, abstractmethod
from dataclasses import field
from AsteriskCommand.commons.asterisk_action import Action


class AsteriskCommand(ABC):
    Action: str
    auxiliar_commands: List
    Variables: Dict = field(default_factory=dict)
    is_asterisk_command: bool = True

    @abstractmethod
    def as_dict(self) -> Dict:
        raise NotImplementedError("Method not implemented")

    def as_json_asterisk_command(self) -> str:
        return json.dumps(self.as_dict())

    def as_asterisk_command(self) -> Action:
        return Action(name=self.Action, keys=self.as_dict(), variables=self.Variables)



from AsteriskCommand.originate import Originate
from AsteriskCommand.atxfer import Atxfer
from unittest import TestCase
from uuid import uuid4


class TestReceiveCommands(TestCase):
    def test_atxfer(self):
        action_id = str(uuid4())
        command_dictionary = {
            "Action": "Atxfer",
            "ActionID": action_id,
            "Channel": "SIP/123",
            "Context": "Test",
            "Exten": "SIP/111",
            "Priority": "1",
            "peerName": "1000",
        }

        asterisk_command = Atxfer(**command_dictionary)

        self.assertEqual(asterisk_command.ActionID, action_id)

    def test_originate(self):
        action_id = str(uuid4())
        transaction_id = str(uuid4())

        command_dictionary = {
            "Action": "Originate",
            "ActionID": action_id,
            "Channel": "SIP/100",
            "Context": "Test",
            "Exten": "999",
            "Priority": "1",
            "Callerid": "10000",
            "Timeout": 30,
            "Variables": {
                "IDRegistro": "100",
                "CallActionID": transaction_id,
                "AgentCode": "999",
            },
        }

        asterisk_command = Originate(**command_dictionary)

        self.assertEqual(asterisk_command.ActionID, action_id)
        self.assertEqual(asterisk_command.Variables["IDRegistro"], "100")



from unittest import TestCase
from uuid import uuid4
from AsteriskCommand.hangup import Hangup
from tests.commands.auxiliar_commands import Auxiliar


class TestCommands(TestCase):
    def test_hangup(self):
        action_id = str(uuid4())

        asterisk_command = Hangup(
            ActionID=action_id, Channel="SIP/999", peerName="1000"
        )

        expected_dictionary = {
            "Action": "Hangup",
            "ActionID": action_id,
            "Channel": "SIP/999",
            "peerName": "1000",
        }

        expected_asterisk_command = Auxiliar.format_as_asterisk_command(
            str(
                f"""Action: Hangup
                ActionID: {action_id}
                Channel: SIP/999
                peerName: 1000
                """
            )
        )

        self.assertDictEqual(asterisk_command.as_dict(), expected_dictionary)
        self.assertEqual(
            str(asterisk_command.as_asterisk_command()), expected_asterisk_command
        )
        self.assertTrue(asterisk_command.is_asterisk_command)



from typing import Dict
from dataclasses import dataclass
from AsteriskCommand.commons.asterisk_command import AsteriskCommand


@dataclass
class Hangup(AsteriskCommand):
    ActionID: str
    Channel: str
    peerName: str
    Action: str = "Hangup"

    def as_dict(self) -> Dict:
        return {
            "Action": self.Action,
            "ActionID": self.ActionID,
            "Channel": self.Channel,
            "peerName": self.peerName,
        }



from unittest import TestCase
from uuid import uuid4
from AsteriskCommand.set_var import SetVar
from tests.commands.auxiliar_commands import Auxiliar


class TestCommands(TestCase):
    def test_set_var(self):
        action_id = str(uuid4())

        asterisk_command = SetVar(
            ActionID=action_id, Channel="SIP/999", Variable="myVar", Value="myValue"
        )

        expected_dictionary = {
            "Action": "SetVar",
            "ActionID": action_id,
            "Channel": "SIP/999",
            "Variable": "myVar",
            "Value": "myValue",
        }

        expected_asterisk_command = Auxiliar.format_as_asterisk_command(
            str(
                f"""Action: SetVar
                ActionID: {action_id}
                Channel: SIP/999
                Variable: myVar
                Value: myValue
                """
            )
        )

        self.assertDictEqual(asterisk_command.as_dict(), expected_dictionary)
        self.assertEqual(
            str(asterisk_command.as_asterisk_command()), expected_asterisk_command
        )
        self.assertTrue(asterisk_command.is_asterisk_command)



from typing import Dict
from dataclasses import dataclass
from AsteriskCommand.commons.asterisk_command import AsteriskCommand


@dataclass
class QueueStatus(AsteriskCommand):
    ActionID: str
    Action: str = "QueueStatus"

    def as_dict(self) -> Dict:
        return {"Action": self.Action, "ActionID": self.ActionID}



from unittest import TestCase
from uuid import uuid4
from AsteriskCommand.get_info_anexos import GetInfoAnexos
from tests.commands.auxiliar_commands import Auxiliar


class TestCommands(TestCase):
    def test_get_info_anexos(self):
        action_id = str(uuid4())

        asterisk_command = GetInfoAnexos(ActionID=action_id)

        expected_dictionary = {
            "Action": "GetInfoAnexos",
            "ActionID": action_id,
        }

        expected_asterisk_command = Auxiliar.format_as_asterisk_command(
            str(
                f"""Action: GetInfoAnexos
                ActionID: {action_id}
                """
            )
        )

        self.assertDictEqual(asterisk_command.as_dict(), expected_dictionary)
        self.assertEqual(
            str(asterisk_command.as_asterisk_command()), expected_asterisk_command
        )
        self.assertFalse(asterisk_command.is_asterisk_command)



from typing import Dict
from dataclasses import dataclass
from AsteriskCommand.commons.asterisk_command import AsteriskCommand


@dataclass
class QueueAdd(AsteriskCommand):
    ActionID: str
    Queue: str
    Interface: str
    Penalty: str
    Paused: str
    MemberName: str
    StateInterface: str
    Action: str = "QueueAdd"

    def as_dict(self) -> Dict:
        return {
            "Action": self.Action,
            "ActionID": self.ActionID,
            "Queue": self.Queue,
            "Interface": self.Interface,
            "Penalty": self.Penalty,
            "Paused": self.Paused,
            "MemberName": self.MemberName,
            "StateInterface": self.StateInterface,
        }



from typing import Dict
from dataclasses import dataclass
from AsteriskCommand.commons.asterisk_command import AsteriskCommand


@dataclass
class SetVar(AsteriskCommand):
    ActionID: str
    Channel: str
    Variable: str
    Value: str
    Action: str = "SetVar"

    def as_dict(self) -> Dict:
        return {
            "Action": self.Action,
            "ActionID": self.ActionID,
            "Channel": self.Channel,
            "Variable": self.Variable,
            "Value": self.Value,
        }



from setuptools import setup, find_packages
import subprocess

git_command_result: subprocess.CompletedProcess = subprocess.run(
    ["git", "describe", "--tags"], capture_output=True, encoding="utf-8"
)
actual_version: str = git_command_result.stdout.strip("\n")

setup(
    name="cl3k-asteriskcommands",
    version=actual_version,
    packages=find_packages(),
    author="Juares Vermelho Diaz (CL3k)",
    author_email="jvermelho@cl3k.com",
    description="API for share asterisk commands beetween the microservices",
    keywords="Asterisk commands",
    install_requires=[],
)
